/*
 *  bionj_cond: A modified version of BIONJ to work with conditioned genome reconstruction
 *  Copyright (C) 2005 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 *  Based on BIONJ:
 * Gascuel O.1997. BIONJ: an improved version of the NJ algorithm based on a simple model of sequence data. Molecular Biology and Evolution, 14:685-695. 
;                                                                           ;
;                         BIONJ program                                     ;
;                                                                           ;
;                         Olivier Gascuel                                   ;
;                                                                           ;
;                         GERAD - Montreal- Canada                          ;
;                         olivierg@crt.umontreal.ca                         ;
;                                                                           ;
;                         LIRMM - Montpellier- France                       ;
;                         gascuel@lirmm.fr                                  ;
;                                                                           ;
;                         UNIX version, written in C                        ;
;                         by Hoa Sien Cuong (Univ. Montreal)                ;
;                                                                           ;

Input file is standard phylip distance matrices as described below in comments, except that for a set of m taxa there should be m distance matrices, each in standard format,  except that for the ith matrix (using taxon i as the conditioning genome), row i and column i are NaN.
Output is a tree in phylip format without edge lengths (because we can't average these over distances from different conditioning genomes)
Usage: bionj_cond [-qnrsh] [-v slfile] infile outfile
Options: 
-q: quiet operation (default)
-n: noisy operation
-r: ties for number of votes broken at random (default)
-s: ties for number of votes broken systematically
-v slfile: votes weighted by approximate inverse variance:number of genes in conditioning genome divided by sum of edge lengths: slfile is file listing conditioning genome sizes (a single row of text with number of genes in each conditioning genome, separated by white space).
-h: help
;
To compile:
gcc -ansi -pedantic -Wall -O2 -o bionj_cond bionj_cond_1.0.c
;
Uses the GNU C library
;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <getopt.h>
#include <ctype.h>

#define PREC 8                             /* precision of branch-lengths  */
#define PRC  100
#define LEN  1000                            /* length of taxon names        */

#define PROGRAM_NAME "bionj_cond"
#define PROGRAM_VERSION "1.0"

typedef struct word
{
  char name[LEN];
  struct word *suiv;
}WORD;

typedef struct pointers
{
  WORD *head;
  WORD *tail;
}POINTERS;

typedef struct aggpt/*pointers to where we aggregated a taxon*/
{
	int a;
	struct aggpt *newnode;
}AGGPT;

void   Initialize(float **delta, FILE *input, int n, POINTERS *trees, int k);

void   Compute_sums_Sx(float **delta, int n, int k);

void   Best_pair_overall(float ***delta,int *a, int *b, int n, float **votes, int tiebreak, int *cgsizes, int r); /*new function for BIONJ_cond*/

void   Best_pair(float **delta, int r, int *a, int *b, int n, int k);

void   Finish(float **delta, int n, POINTERS *trees, FILE *output);

void   Concatenate(char chain1[LEN], int ind, POINTERS *trees, int post);

void   Print_output(int i, POINTERS *trees, FILE *output);

float Distance(int i, int j, float **delta);

float Variance(int i, int j, float **delta);

float Sum_S(int i, float **delta);

float Agglomerative_criterion(int i, int j, float **delta, int r);

float Branch_length(int a, int b, float **delta, int r);

float Reduction4(int a, float la, int b, float lb, int i, float lamda,
		 float **delta);

float Reduction10(int a, int b, int i, float lamda, float vab, float
		  **delta);
float Lamda(int a, int b, float vab, float **delta, int n, int r, int k);

float Finish_branch_length(int i, int j, int k, float **delta);

int    Emptied(int i, float **delta);

int    Symmetrize(float **delta, int n, int k);

int Get_r(float ** delta, int n, int k); /*new function for BIONJ_cond*/

int Get_newa(int a, AGGPT *aggptrs);/*new function for BIONJ_cond*/
 
void Set_newa(int *newa, int a, AGGPT *aggptrs, float ***delta, int n); /*new function for BIONJ_cond*/

float get_var_weight(float **delta, int n, int k, int *cgsizes); /*new function for BIONJ_cond*/

void get_cgsizes(int *cgsizes, int n, FILE *slfile); /*new function for BIONJ_cond*/

void Resolve_best_pair(float ***delta,int *i,int *j,int n,int k,int tiebreak, int *cgsizes); /*new function for BIONJ_cond*/

float Voteplus(float ***delta,int n,int k, int *cgsizes, int tiebreak);/*new function for BIONJ_cond*/
/*TEST FUNCTIONS for BIONJ_cond***********************/
void print_distcheck(int n, float ***delta);

void display_tree(int n, int r, POINTERS *trees);

void display_subtree(int i, POINTERS *trees);

void print_abchoices(int *newa, int*newb, int n);
/*******************************************************/

enum {
	systbreak, 
	randbreak,
	varbreak
};


/*;;;;;;;;;;;  INPUT, OUTPUT, INITIALIZATION ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                           ;
;                                                                           ;
;              The delta matrix is read from the input-file.                ;
;              It is recommended to put it and the executable in            ;
;              a special directory. The input-file and output-file          ;
;              can be given as arguments to the executable by               ;
;              typing them after the executable (Bionj input-file           ;
;              output-file) or by typing them when asked by the             ;
;              program. The input-file has to be formated according         ;
;              the PHYLIP standard. The output file is formated             ;
;              according to the NEWSWICK standard.                          ;
;                                                                           ;
;              The lower-half of the delta matrix is occupied by            ;
;              dissimilarities. The upper-half of the matrix is             ;
;              occupied by variances. The first column                      ;
;              is initialized as 0; during the algorithm some               ;
;              indices are no more used, and the corresponding              ;
;              positions in the first column are set to 1.                  ;
;                                                                           ;
;              This delta matix is made symmetrical using the rule:         ;
;              Dij = Dji <- (Dij + Dji)/2. The diagonal is set to 0;        ;
;              during the further steps of the algorithm, it is used        ;
;              to store the sums Sx.                                        ;
;                                                                           ;
;              A second array, trees, is used to store taxon names.         ;
;              During the further steps of the algoritm, some               ;
;              positions in this array are emptied while the others         ;
;              are used to store subtrees.                                  ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/


/*;;;;;;;;;;;;;;;;;;;;;;;;;; Initialize        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
; Description : This function reads an input file and return the            ;
;               delta matrix and trees: the list of taxa.                   ;
;                                                                           ;
; input       :                                                             ;
;              float **delta : delta matrix                                 ;
;              FILE *input    : pointer to input file                       ;
;              int n          : number of taxa                              ;
;              char **trees   : list of taxa                                ;
;              int k               : which conditioning genome
;                                                                           ;
; return value:                                                             ;
;              float **delta : delta matrix                                 ;
;              char *trees    : list of taxa                                ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

void Initialize(float **delta, FILE *input, int n, POINTERS *trees, int k)
{
  int lig;                                          /* matrix line       */
  int col;                                          /* matrix column     */
  int dummy; /*number of taxa, which we already know*/
  float distance;
  char name_taxon[LEN];                             /* taxons name      */
  WORD *name;
  
  name=NULL;
  fscanf(input,"%d",&dummy); /* read line with number of taxa*/
  for(lig=1; lig <= n; lig++)
    {
      fscanf(input,"%s",name_taxon);                  /* read taxons name */
      
      
      if(k==1) /*only need to do this for first conditioning genome*/
      {
      	name=(WORD *)calloc(1,sizeof(WORD));            /* taxons name is   */
      	if(name == NULL)                                /* put in trees      */
		{
	  	printf("Out of memories !!");
	  	exit(0);
		}
      	else
		{
	  	strcpy(name->name,name_taxon);
	  	name->suiv=NULL;
	  	trees[lig].head=name;
	  	trees[lig].tail=name;
		}
	}
	  for(col= 1; col <= n; col++)
	    {
	      fscanf(input,"%f",&distance);             /* read the distance  */
	      delta[lig][col]=distance;
	    }

    }
}

/*;;;;;;;;;;;;;;;;;;;;;;;;;;; Print_output;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;                                                                           ;
; Description : This function prints out the tree in the output file.       ;
;                                                                           ;
; input       :                                                             ;
;              POINTERS *trees : pointer to the subtrees.                   ;
;              int i          : indicate the subtree i to be printed.       ;
:              FILE *output   : pointer to the output file.                 ;
;                                                                           ;
; return value: The phylogenetic tree in the output file.                   ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/


void Print_output(int i, POINTERS *trees, FILE *output)
{
  WORD *parcour;
  parcour=trees[i].head;
  while(parcour != NULL)
    {
      fprintf(output,"%s",parcour->name);
      parcour=parcour->suiv;
    }
}




/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                           ;
;                         Main program                                      ;
;                                                                           ;
;                         argc is the number of arguments                   ;
;                         **argv contains the arguments:                    ;
;                         the first argument has to be BIONJ;               ;
;                         the second is the inptu-file;                     ;
;                         the third is the output-file.                     ;
;                         When the input and output files are               ;
;                         not given, the user is asked for them.            ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/


int main(int argc, char **argv)
{
  FILE *input;                            /* pointer to input file       */
  FILE *output;                           /* pointer to output file      */
  FILE *slfile=NULL;				/*pointer to file with conditioning genome sizes */
  POINTERS *trees;                        /* list of subtrees            */
  char *Name_fich1;                       /* name of the input file      */
  char *Name_fich2;                       /* name of the output file     */
  char *chain1;                           /* stringized branch-length    */
  char *chain2;                           /* idem                        */
  int *a, *b;                             /* pair to be agglomerated     */
  float ***delta;                          /* delta matrix set                */
  float la;                               /* first taxons branch-length */
  float lb;                               /* second taxons branch-length*/
  float vab;                              /* variance of Dab             */
  float lamda;
  int i, k;
  int r;                                  /* number of subtrees          */
  int n;                                  /* number of taxa              */
  int x, y;
  clock_t clock_start, clock_end;
  double t;
  float **votes;                       /*number of votes for each pair to agglomerate: float so we can use random numbers to break ties */
  int rk;                               /*number of nodes remaining in a given matrix*/
  int tiebreak;                        /*ties broken systematically or randomly or using variances*/
  int silent;                            /*no output to screen*/
  AGGPT *aggptrs;
  int *newa;
  int *newb;
 int c;
 char *slpt=NULL;
 int * cgsizes;
   
 
  clock_start=clock();
  
	silent=1;
	tiebreak=randbreak;/*ties broken at random by default*/     
       opterr = 0;
         while ((c = getopt (argc, argv, "qnrsv:h")) != -1)
         switch (c) {
	case 'q':
		silent = 1; /*silent operation except for error messages (default)*/
	break;
	case 'n':
		silent = 0; /*noisy operation*/
	break;
	case 'r':    
		tiebreak = randbreak;/*break ties at random (default)*/
	break;
	case 's':
		tiebreak = systbreak;/*break ties systematically*/ 
	break;
	case 'v':
		tiebreak = varbreak;/*weight using variances*/ 
 		slpt=optarg;
	break;
	case 'h':
	  printf("\n%s %s\n",PROGRAM_NAME,PROGRAM_VERSION); 
	  printf("Modified BIONJ for conditioned genome reconstruction\n"); 
	  printf("Usage: ");
	  printf(PROGRAM_NAME);
	  printf(" [-qnrs] [-v slfile] [infile] [outfile]\n");
	  printf("Options: \n");
	  printf("-q: quiet operation (default)\n");
	  printf("-n: noisy operation: reports the options selected and the pair aggregated each step\n");
	  printf("-r: ties for number of votes broken at random (default)\n");
	  printf("-s: ties for number of votes broken systematically (choose the first pair with max number of votes, moving across then down the array of pairs\n");
	  printf("-v slfile: votes weighted by approximate inverse variances: number of genes in conditioning genome divided by sum of edge lengths : slfile is file listing conditioning genome sizes (a single row of text with number of genes in each conditioning genome, separated by white space).\n");
	  printf("-h: this help message\n");
	  printf("Input file format: for m taxa, a set of m distance matrices in a single PHYLIP file\n");
	  printf("The ith distance matrix should be distances estimated using conditioning genome i,\n");
	  printf("with ith row and column NaN\n");
	  printf("Output file format: a topology with no edge lengths for all m taxa in PHYLIP format \n");
	  printf("Based on BIONJ.  Please cite Gascuel O. 1997. BIONJ: an improved version of the NJ algorithm based on a simple model of sequence data. Molecular Biology and Evolution, 14:685-695.\n");
	  printf("Copyright (C) 2005 by Matthew Spencer, Department of Mathematics and Statistics, Dalhousie University (m.spencer@liverpool.ac.uk).\n");
	   return 1;

	break;
	case '?':
             if (isprint (optopt))
               fprintf (stderr, "Unknown option `-%c'.\n", optopt);
             else
               fprintf (stderr,
                        "Unknown option character `\\x%x'.\n",
                        optopt);
             return 1;
	default:
             abort ();
	}


  /*   Allocation of memories    */
  
  Name_fich1=(char*)calloc(LEN,sizeof(char));
  Name_fich2=(char*)calloc(LEN,sizeof(char));
  a=(int*)calloc(1,sizeof(int));
  b=(int*)calloc(1,sizeof(int));
  chain1=(char *)calloc(LEN,sizeof(char));
  chain2=(char *)calloc(LEN,sizeof(char));
    

if (!(silent==1))
{
	/*******cite original BIONJ**********************/
	printf("BIONJ_cond for conditioned genome reconstruction\n");
	printf("Based on BIONJ.  Please cite Gascuel O. 1997. BIONJ: an improved version of the NJ algorithm based on a simple model of sequence data. Molecular Biology and Evolution, 14:685-695.\n");
   	printf("This program is free software; you may redistribute it and/or modify it\n");
	printf("under the terms of the GNU General Public License as published by the Free\n");
   	printf("Software Foundation; either version 2 of the License, or (at your option)\n");
   	printf("any later version.\n");
	printf("This program is distributed in the hope that it will be useful, but\n");
	printf("WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n");
   	printf("or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\n");
	switch (tiebreak) {
		case systbreak:
			printf("breaking ties systematically\n");
		break;
		case randbreak:
			printf("breaking ties at random\n");
		break;
		case varbreak:
			printf("weighting by approximate inverse variances\n");
	}
/**********************************************/  
}  
    
  if(argc > optind)
    {
      input= fopen(argv[optind],"r");
      if(input==NULL)
	{
	  printf("\ninput file not found !!");
	  while(input==NULL)
	    {
	      printf("\nEnter the input file : ");
	      scanf("%s",Name_fich1);                     /* ask for the input-file */
	      input=fopen(Name_fich1,"r");
	    }
	}
      
    }
  else
    {
      do                                        /* ask for the input-file */
	{                                       
	  printf("\nEnter input file : ");
	  scanf("%s",Name_fich1);
	  input=fopen(Name_fich1,"r");
	}while(input==NULL);
  }
  output=NULL;
  if(argc>(optind+1))
 {
 	output= fopen(argv[optind+1],"w");
 }
 else
 {
      do   /* ask for ouput-file.        */
	{
	  printf("\nEnter output file : ");
	  scanf("%s",Name_fich2);
	  output=fopen(Name_fich2,"w");
	}while(output==NULL);
    }
 
    
  if(tiebreak==varbreak)  {
	slfile=fopen(slpt,"r");/*open the conditioning genome size file*/
	if(slfile==NULL) {
		printf("\nconditioning genome size file not found");
		do {                                        
			printf("\nEnter conditioning genome size file : ");
			scanf("%s",slpt);
			slfile=fopen(slpt,"r");
		}while(slfile==NULL);
	}
}    

fscanf(input,"%d",&n);/*number of taxa in each matrix*/
  /*      Create the delta matrix and the votes matrix     */
  delta=(float***)calloc(n+1,sizeof(float**));/*array of delta matrices for each conditioning genome*/
  votes=(float **)calloc(n+1,sizeof(float*)); /*array with number of votes for each pair */
  cgsizes=(int *)calloc(n+1,sizeof(int)); /*array with number of genes in conditioning genome*/
  for(k=0;k<=n;k++)/*k=0 is used for global list of what's been aggregated: wasteful of memory but requires fewest changes to code*/
  {
  	delta[k]=(float **)calloc(n+1,sizeof(float*));
  	for(i=1; i<= n; i++)
    	{
      		delta[k][i]=(float *)calloc(n+1, sizeof(float));
      		if(delta[k][i] == NULL)
		{
	  	printf("Out of memories!!");
	  	exit(0);
		}
    	}
}

 for(k=1;k<=n;k++)
{
	votes[k]=(float *)calloc(n+1, sizeof(float));
	if(votes[k] == NULL)
	{
	  	printf("Out of memories!!");
	  	exit(0);
	}
}

  trees=(POINTERS *)calloc(n+1,sizeof(POINTERS));
  if(trees == NULL)
    {
      printf("Out of memories!!");
      exit(0);
    }
  
    aggptrs=(AGGPT *)calloc(n+1,sizeof(AGGPT));
    if(aggptrs == NULL)                                
		{
	  	printf("Out of memories !!");
	  	exit(0);
		}
    for(i=1;i<=n;i++)
   {
	aggptrs[i].a=i;
	aggptrs[i].newnode=NULL;
   }

  newa=(int*)calloc(n+1,sizeof(int));
  if(newa==NULL)
		{
	  	printf("Out of memories !!");
	  	exit(0);
		}
  newb=(int*)calloc(n+1,sizeof(int));
  if(newb==NULL)
		{
	  	printf("Out of memories !!");
	  	exit(0);
		}
if(tiebreak==varbreak){  
	get_cgsizes(cgsizes, n, slfile)  ; /*read in the size of each conditioning genome*/
}  
  /*   initialise and symmetrize the running delta matrix    */
  
      r=n;
      *a=0;
      *b=0;
      la=0;
      lb=0;
      rewind(input);
      for (k=1;k<=n;k++)
      {
		Initialize(delta[k], input, n, trees, k);
		Symmetrize(delta[k], n, k);
      }
      
      srand( (unsigned)time( NULL ) );
  
      
      while (r > 3)                             /* until r=3                 */
	{
	  for (k=1;k<=n;k++)
	  {
	  	Compute_sums_Sx(delta[k], n, k);             /* compute the sum Sx       */
	   }
	  
/*	  print_distcheck(n, delta);*/ /*test function*/
	   

	 Best_pair_overall(delta,a,b,n,votes,tiebreak,cgsizes,r);  /*find the best pair to join over all conditioning genomes */
	 if (!silent)
	 {
	 printf("step %d, aggregating pair %d, %d\n",n-r+1,*a,*b);
	 }
	   /*are the chosen taxa already emptied: if so, find where they went*/
 	  Set_newa(newa,*a, aggptrs, delta,n);
 	  Set_newa(newb,*b, aggptrs, delta,n);
/*	  print_abchoices(newa, newb, n);  */
	  
	  for (k=1;k<=n;k++)
	  {
	  if(!(k==newa[k]) && !(k==newb[k]))/*no change if the conditioning genome is one of the pair*/
	  	{
	  		la=0;
			lb=0;
			vab=Variance(newa[k], newb[k], delta[k]);
			rk=Get_r(delta[k],n,k); /*number of nodes left in this matrix*/
	  		la=Branch_length(newa[k], newb[k], delta[k], rk);    /* compute branch-lengths  */
	  		lb=Branch_length(newb[k], newa[k], delta[k], rk);    /* using formula (2)      */
			lamda=Lamda(newa[k], newb[k], vab, delta[k], n, rk, k); /* compute lambda* using (9)*/
			for(i=1; i <= n; i++)
	    		{
	      		if(!Emptied(i,delta[k]) && (i != newa[k]) && (i != newb[k]))
				{
		  		if(newa[k] > i)
		    		{
		      			x=newa[k];
		      			y=i;
		    		}
		  		else
		    		{
		      			x=i;
		      			y=newa[k];                           /* apply reduction formulae */
		    		}                                  /* 4 and 10 to delta        */
		  		delta[k][x][y]=Reduction4(newa[k], la, newb[k], lb, i, lamda, delta[k]);
				delta[k][y][x]=Reduction10(newa[k], newb[k], i, lamda, vab, delta[k]);
		  		
				}
	    		}
	    }
}

/***********************************************/	    
/*TEST CODE**********************************/
/*		display_tree(n, r, trees);*/
/***********************************************/	    
	    
	    	    
	  strcpy(chain1,"");                     /* agglomerate the subtrees */
	  strcat(chain1,"(");                    /* a and b together with the*/
	  Concatenate(chain1, newa[0], trees, 0);     /* branch-lengths according */
	  strcpy(chain1,"");                     /* to the NEWSWICK format   */
/******skip the edge lengths from output file because they have no meaning*/	  
/*	  strcat(chain1,":");
	  sprintf(chain1+strlen(chain1),"%f",la);*/
/**************************************************************************/	  
  
/* 	  gcvt(la,PREC, chain2); */
/* 	  strcat(chain1, chain2); */
	  strcat(chain1,",");
	  Concatenate(chain1,newa[0], trees, 1);
	  trees[newa[0]].tail->suiv=trees[newb[0]].head;
	  trees[newa[0]].tail=trees[newb[0]].tail;
	  strcpy(chain1,"");

/******skip the edge lengths from output file because they have no meaning*/	  
/*	  strcat(chain1,":");
	  sprintf(chain1+strlen(chain1),"%f",lb);*/
/**************************************************************************/	  
/* 	  gcvt(lb, PREC, chain2); */
/* 	  strcat(chain1, chain2); */
	  strcat(chain1,")");
	  Concatenate(chain1, newa[0], trees, 1);
	
	  for(k=0;k<=n;k++) /*empty b line in all layers of distance matrix, including global list in k=0 */
	  {
	  	
		if(k!=*a && k!=*b) /*no emptying if pair member is conditioning genome*/
		
		
  			delta[k][newb[0]][0]=1.0;                     /* make the b line empty     */
	  }
	  trees[newb[0]].head=NULL;
	  trees[newb[0]].tail=NULL;
	  aggptrs[newb[0]].newnode=&aggptrs[newa[0]];    /*keep a record of where the aggregated taxon went*/
/*	printf("b went to: %d\n",aggptrs[newb[0]].newnode->a);*/
	  
	  r=r-1;                                /* decrease global r                */
	  
/*	  print_distcheck(n, delta);*/
	  
	}	  
	
	/**********************/
	
	/************k=0 is the global list of pairs, and we don't care about edge lengths********/
	Finish(delta[0], n, trees, output);
	/*************************/	
	
      for(i=1; i<=n; i++)       	          /* of the last three subtrees*/
	{				                /* and print the tree in the */
	  for (k=1;k<=n;k++)
	  {
	  	delta[k][i][0]=0.0;		          /* output-file               */
	   }
	  trees[i].head=NULL;
	  trees[i].tail=NULL;
	}
	/**************************/
	
  free(delta);
  free(votes);
  free(cgsizes);
  free(trees);
  free(aggptrs);
  free(newa);
  free(newb);
  fclose(input);
  fclose(output);
  if(tiebreak==varbreak) fclose(slfile);

  clock_end=clock(); i=(int)(t=(clock_end*1.0-clock_start)/CLOCKS_PER_SEC);
/*   printf("\nTime used  %d:%d:%.3f\n", i/3600,(i%3600)/60, t-(i/60)*60); */
  
  if(!(silent==1))
    printf("?Time %f\n",t);
  return 0;
}

/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;                             Utilities                                     ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/



/*;;;;;;;;;;;;;;;;;;;;;;;;;;; Symmetrize  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
; Description : This function verifies if the delta matrix is symmetric;    ;
;               if not the matrix is made symmetric.                        ;
;                                                                           ;
; input       :                                                             ;
;              float **delta : delta matrix                                 ;
;              int n          : number of taxa                              ;
;              int k          : conditioning genome (skip this row and col) ;
;                                                                           ;
; return value:                                                             ;
;              int symmetric  : indicate if the matrix has been made        ;
;                               symmetric or not                            ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

int Symmetrize(float **delta, int n, int k)
{
  int lig;                                         /* matrix line        */
  int col;                                         /* matrix column      */
  float value;                                     /* symmetrized value  */
  int symmetric;
  
  symmetric=1;
  for(lig=1; lig  <=  n; lig++)
  if (!(lig==k))
    {
      for(col=1; col< lig; col++)
      if(!(col==k))
	{
	  if(delta[lig][col] != delta[col][lig])
	    {
	      value= (delta[lig][col]+delta[col][lig])/2;
	      delta[lig][col]=value;
	      delta[col][lig]=value;
	      symmetric=0;
	    }
        }
    }
	if (!symmetric)	
	printf("\n Matrix %d  is not symmetric.\n ",k);
  return(symmetric);
}




/*;;;;;;;;;;;;;;;;;;;;;;;;;;; Concatenate ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;                                                                           ;
; Description : This function concatenates a string to another.             ;
;                                                                           ;
; input       :                                                             ;
;      char *chain1    : the string to be concatenated.                     ;
;      int ind         : indicate the subtree to which concatenate the      ;
;                        string                                             ;
;      POINTERS *trees  : pointer to subtrees.                              ;
;      int post        : position to which concatenate (front (0) or        ;
;                        end (1))                                           ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

void Concatenate(char chain1[LEN], int ind, POINTERS *trees, int post)
{
  WORD *bran;

  
  bran=(WORD *)calloc(1,sizeof(WORD));
  if(bran == NULL)
    {
      printf("Out of memories");
      exit(0);
    }
  else
    {
      strcpy(bran->name,chain1);
      
      bran->suiv=NULL;
      

    }
  if(post == 0)
    {
      bran->suiv=trees[ind].head;
      trees[ind].head=bran;
    }
  else
    {
      
      trees[ind].tail->suiv=bran;
      trees[ind].tail=trees[ind].tail->suiv;
    }
}


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Distance;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
; Description : This function retrieve ant return de distance between taxa  ;
;               i and j from the delta matrix.                              ;
;                                                                           ;
; input       :                                                             ;
;               int i          : taxon i                                    ;
;               int j          : taxon j                                    ;
;               float **delta : the delta matrix                            ;
;                                                                           ;
; return value:                                                             ;
;               float distance : dissimilarity between the two taxa         ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

float Distance(int i, int j, float **delta)
{
  if(i > j)
    return(delta[i][j]);
  else
    return(delta[j][i]);
}


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Variance;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
; Description : This function retrieve and return the variance of the       ;
;               distance between i and j, from the delta matrix.            ;
;                                                                           ;
; input       :                                                             ;
;               int i           : taxon i                                   ;
;               int j           : taxon j                                   ;
;               float **delta  : the delta matrix                           ;
;                                                                           ;
; return value:                                                             ;
;               float distance : the variance of  Dij                       ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

float Variance(int i, int j, float **delta)
{
  if(i > j)
    return(delta[j][i]);
  else
    return(delta[i][j]);
}


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Emptied ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
; Description : This function verifie if a line is emptied or not.          ;
;                                                                           ;
; input       :                                                             ;
;               int i          : subtree (or line) i                        ;
;               float **delta : the delta matrix                            ;
;                                                                           ;
; return value:                                                             ;
;               0              : if not emptied.                            ;
;               1              : if emptied.                                ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

int Emptied(int i, float **delta)      /* test if the ith line is emptied */
{
  return((int)delta[i][0]);
}


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Sum_S;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;  Description : This function retrieves the sum Sx from the diagonal       ;
;                of the delta matrix.                                       ;
;                                                                           ;
;  input       :                                                            ;
;               int i          : subtree i                                  ;
;               float **delta : the delta matrix                            ;
;                                                                           ;
;  return value:                                                            ;
;                float delta[i][i] : sum Si                                 ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

float Sum_S(int i, float **delta)          /* get sum Si form the diagonal */
{
  return(delta[i][i]);
}


/*;;;;;;;;;;;;;;;;;;;;;;;Compute_sums_Sx;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
; Description : This function computes the sums Sx and store them in the    ;
;               diagonal the delta matrix.                                  ;
;                                                                           ;
; input       :                                                             ;
;     	         float **delta : the delta matrix.                      ;
;     	         int n          : the number of taxa                    ;
;		int k           : the conditioning genome            ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

void Compute_sums_Sx(float **delta, int n, int k)
/*modified for BIONJ_cond: don't include row and col corresponding to conditioning genome in the sum*/
{
  float sum;
  int i;
  int j;
  
  sum=0;
  for(i= 1; i <= n ; i++)
  	if(!(i==k)) /*my modification*/
	{
      		if(!Emptied(i,delta))
			{
	  		sum=0;
	  		for(j=1; j <=n; j++) 
			if(!(j==k)) /*my modification*/
	    			{
	      				if(i != j && !Emptied(j,delta))           /* compute the sum Si */
					sum=sum + Distance(i,j,delta);
	    			}
			}
      		delta[i][i]=sum;                           /* store the sum Si in */
	}
}


/********************Best_pair_overall**********************/
/*Find the best pair to agglomerate over all conditioning genomes
New function for BIONJ_cond
Input
	float ***delta: set of all distance matrices
	int *a : first member of pair
	int *b : second member of pair
	int n: number of taxa
	float **votes : array with number of votes for each pair: float so we can use random numbers to break ties
	int tiebreak: if randbreak, break ties at random.  If systbreak, break ties systematically: first pair with max score, moving across then down the array
	If varbreak, weight votes by approximate inverse variances
	int *cgsizes: number of genes in each conditioning genome
	int r: overall number of taxa remaining in global tree
**********************************************************/

void   Best_pair_overall(float ***delta,int *a, int *b, int n, float **votes, int tiebreak, int *cgsizes, int r)
{
int i,j,k,maxi,maxj,rk;
int *pa, *pb;
float maxval;

pa=(int*)calloc(1,sizeof(int));
pb=(int*)calloc(1,sizeof(int));

if((pa == NULL) | (pb == NULL))
{
  	printf("Out of memories!!");
  	exit(0);
}


*pa=0;
*pb=0;
for (i=1;i<=n;i++)/*set votes to 0 for all pairs*/
{
	for(j=1;j<=n;j++)
	{
	votes[i][j]=0.0;
	}
}
for (k=1;k<=n;k++)
	{
	rk=Get_r(delta[k],n,k);
	if(rk>3)/*more than three nodes not aggregated in this matrix*/
		{
		Best_pair(delta[k], rk, pa, pb, n, k);
		if (rk==r-1) Resolve_best_pair(delta,pa,pb,n,k,tiebreak,cgsizes);/*we haven't yet aggregated taxon k, so need to check how this resolves*/
		votes[*pa][*pb]+=Voteplus(delta,n,k,cgsizes,tiebreak);/*add weighted sum to votes*/

/********TEST CODE********************/
/*			for(i=1;i<=n;i++)
			{
				for(j=1;j<=n;j++)
				{
					printf("%f ",votes[i][j]);
				}	
				printf("\n");
			}
			printf("conditioning on %d, chose pair %d, %d\n\n",k,*pa,*pb);*/
/*****************************************/
		}
	}
/*now set a and b to match the most frequent choice over all k*/


/*****choosing pair with most votes, ties broken systematically by
picking the first entry with max votes as we move across then down the array, or at random*/
maxi=0;
maxj=0;
maxval=0.0;
		for (i=1;i<=n;i++)
		{
			for (j=1;j<=n;j++)
			{
				if (tiebreak==randbreak)
					votes[i][j]+=((float) rand()/(RAND_MAX))/2; /*a random float between 0 and 1/2 breaks ties*/
				if (votes[i][j]>maxval)
				{
					maxval=votes[i][j];
					maxi=i;
					maxj=j;
				}
			}
		}
		
*a=maxi;
*b=maxj;
free(pa);
free(pb);

/********TEST CODE********************/
/*for(i=1;i<=n;i++)
{
	for(j=1;j<=n;j++)
	{
		printf("%f ",votes[i][j]);
	}	
	printf("\n");
}
printf("\n");
printf("overall, chose a=%d, b=%d\n\n",*a,*b);*/
/***************************************/
}



/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Best_pair;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;  Description : This function finds the best pair to be agglomerated by    ;
;                minimizing the agglomerative criterion (1).                ;
;                                                                           ;
;  input       :                                                            ;
;                float **delta : the delta matrix                           ;
;                int r          : number of subtrees                        ;
;                int *a         : contain the first taxon of the pair       ;
;                int *b         : contain the second taxon of the pair      ;
;                int n          : number of taxa                            ;
;                int k          : conditioning genome                  ;
;  return value:                                                            ;
;                int *a         : the first taxon of the pair               ;
;                int *b         : the second taxon of the pair              ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

void Best_pair(float **delta, int r, int *a, int *b, int n, int k)


{
  float Qxy;                         /* value of the criterion calculated*/
  int x,y;                           /* the pair which is tested         */
  float Qmin;                        /* current minimun of the criterion */
  
  Qmin=1.0e300;
  for(x=1; x <= n; x++)
    {
    if (!(x==k)) /*not the conditioning genome */
    {
      if(!Emptied(x,delta))
        {
	  for(y=1; y < x; y++)
	  if (!(y==k)) /*not the conditioning genome */
	    {
	      if(!Emptied(y,delta))
		{
		  Qxy=Agglomerative_criterion(x,y,delta,r); 
/*		  printf("r: %d, k: %d, x: %d, y: %d, Qxy: %f\n", r, k, x, y, Qxy);*/
		  if(Qxy < Qmin-0.000001)
		    {
		      Qmin=Qxy;
		      *a=x;          
		      *b=y;
		    }
		}  
	    }
	  }  
        }
    }
  }

/********************Resolve_best_pair**********************/
/*If taxon k not yet aggregated, we don't know whether the best pair (i,j) from matrix k really means (i,j), (i,k) or (j,k)
Resolve this by checking all matrices where i, j and k are present, and deciding which is the best pair out of these
New function for BIONJ_cond
Input
	float ***delta: set of all distance matrices
	int *i : first member of pair chosen from matrix k
	int *j : second member of pair chosen from matrix k
	int n: number of taxa
	int k: conditioning genome
	int tiebreak: how we weight contribution from each matrix
	int *cgsizes: number of genes in each conditioning genome
We modify *i, *j to reflect this
**********************************************************/
void Resolve_best_pair(float ***delta,int *i,int *j, int n, int k, int tiebreak, int *cgsizes)
{
	int kprime,rkprime;
	float Qij,Qik,Qjk;
	float Sij,Sik,Sjk;
	float v;

	Sij=0.0;
	Sik=0.0;
	Sjk=0.0;
	for (kprime=1;kprime<=n;kprime++) {
		if(kprime!=k && kprime!=*i && kprime!=*j) {/*examine all matrices where i,jk all present*/
			/*in each such matrix kprime, calculate S^(kprime)_{r|s} for r,s=i,j,k, r not equal to s*/
			rkprime=Get_r(delta[kprime],n,kprime);
			Qij=Agglomerative_criterion(*i,*j,delta[kprime],rkprime);
			Qik=Agglomerative_criterion(*i,k,delta[kprime],rkprime);
			Qjk=Agglomerative_criterion(*j,k,delta[kprime],rkprime);
			/*symmetrize to get S^(kprime)_{rs} and add to weighted sum S_{rs}*/
			v=Voteplus(delta,n,kprime,cgsizes,tiebreak);/*how much does this vote count?*/
			if(Qij<=Qik) Sij+=v;
			if(Qij<=Qjk) Sij+=v;
			if(Qik<=Qjk) Sik+=v;
			if(Qik<=Qij) Sik+=v;
			if(Qjk<=Qik) Sjk+=v;
			if(Qjk<=Qij) Sjk+=v;
		}
	}
	/*max of this sum over r,s is the resolved pair*/
	if(Sik>Sij && Sik>Sjk) *j=k;/*ik is best: no change in *i but switch *j to k*/
	else if(Sjk>Sij && Sjk>Sik){/*jk is best*/
		*i=*j;
		*j=k;
	}
	/*otherwise ij is best and no change required*/
}

/***************Voteplus************************/
/*Weighted vote for a pair
Inputs: 
float ***delta: the distance matrix
int n: the number of taxa
int k: which conditioning genome we are working with
int *cgsizes: number of genes in each conditioning genome
int tiebreak: now we weight votes*/
/***********************************************/
float Voteplus(float ***delta,int n,int k, int *cgsizes, int tiebreak)
{
	float v;
	switch (tiebreak) {
		case systbreak:
		case randbreak:
				v=1.0;/*add one vote*/
		break;
		case varbreak:/*add vote weighted by inverse of variance*/
			v=get_var_weight(delta[k],n,k,cgsizes);
		break;
		default:
			v=0.0;/*default case: weight zero*/
	}
	return(v);
}


/*;;;;;;;;;;;;;;;;;;;;;;Finish_branch_length;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;  Description :  Compute the length of the branch attached                 ;
;                 to the subtree i, during the final step                   ;
;                                                                           ;
;  input       :                                                            ;
;                int i          : position of subtree i                     ;
;                int j          : position of subtree j                     ;
;                int k          : position of subtree k                     ;
;                float **delta :                                            ;
;                                                                           ;
;  return value:                                                            ;
;                float length  : The length of the branch                   ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/

float Finish_branch_length(int i, int j, int k, float **delta)
{
  float length;
  length=0.5*(Distance(i,j,delta) + Distance(i,k,delta)
	      -Distance(j,k,delta));
  return(length);
}


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Finish;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;
;  Description : This function writes the tree to the output file
(in original BIONJ, it also computed the edge lengths for the last three subtrees)

;  input       :                                                            ;
;                float **delta  : the delta matrix                          ;
;                int n           : the number of taxa                       ;
;                WORD *trees   : list of subtrees                           ;
;                                                                           ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/
void Finish(float **delta, int n, POINTERS *trees, FILE *output)
{
  int l=1;
  int i=0;
/*  char *str;*/
/*float length;*/
  WORD *bidon;
  WORD *ele;
  int last[3];                            /* the last three subtrees     */
    
  /*str=(char *)calloc(LEN,sizeof(char));
  
  if(str == NULL)
    {
      printf("Out of memories !!");
      exit(0);
    }*/

  while(l <= n)
    {                                       /* find the last tree subtree  */
      if(!Emptied(l, delta))
	{
	  last[i]=l;
	  i++;
	}
      l++;
    }

/*  length=Finish_branch_length(last[0],last[1],last[2],delta);*/
  fprintf(output,"(");
  Print_output(last[0],trees,output);
  
/***********skip the edge lengths******/  
/*  fprintf(output,":");
  fprintf(output,"%f,",length);*/
  fprintf(output,",");
/**************************************/ 
 
/*  length=Finish_branch_length(last[1],last[0],last[2],delta);*/
  Print_output(last[1],trees,output);
/***********skip the edge lengths******/  
/*  fprintf(output,":");
  fprintf(output,"%f,",length);*/
  fprintf(output,",");
/**************************************/ 
  
  /*length=Finish_branch_length(last[2],last[1],last[0],delta);*/
  Print_output(last[2],trees,output);
/***********skip the edge lengths******/  
/*  fprintf(output,":");
  fprintf(output,"%f",length);*/
/**************************************/  
  fprintf(output,");");
  fprintf(output,"\n");
  
  
  for(i=0; i < 3; i++)
    {
      bidon=trees[last[i]].head;
      ele=bidon;
      while(bidon!=NULL)
	{
	  ele=ele->suiv;
	  free(bidon);
	  bidon=ele;
	}
    }
  /*free(str);*/
}


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*\
;                                                                           ;   
;                          Formulae                                         ;
;                                                                           ;  
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/


float Agglomerative_criterion(int i, int j, float **delta, int r)
{
  float Qij;
  Qij=(r-2)*Distance(i,j,delta)                           /* Formula (1) */
    -Sum_S(i,delta)
    -Sum_S(j,delta); 
  
  return(Qij);                       
}


float Branch_length(int a, int b, float **delta, int r)
{
  float length;
  length=0.5*(Distance(a,b,delta)                         /* Formula (2) */
	      +(Sum_S(a,delta)
		-Sum_S(b,delta))/(r-2)); 
  return(length);                                   
}


float Reduction4(int a, float la, int b, float lb, int i, float lamda,
		 float **delta)
{
  float Dui;
  Dui=lamda*(Distance(a,i,delta)-la)
    +(1-lamda)*(Distance(b,i,delta)-lb);                /* Formula (4) */
  return(Dui);
}


float Reduction10(int a, int b, int i, float lamda, float vab,
		  float **delta)
{
  float Vci;
  Vci=lamda*Variance(a,i,delta)+(1-lamda)*Variance(b,i,delta)
    -lamda*(1-lamda)*vab;                              /*Formula (10)  */
  return(Vci);
}

float Lamda(int a, int b, float vab, float **delta, int n, int r, int k)
{
/***modified for BIONJ_cond: don't include conditioning genome in sum
k is the conditioning genome
***/
  float lamda=0.0;
  int i;
  
  if(vab==0.0)
    lamda=0.5;
  else
    {
      for(i=1; i <= n ; i++)
	{
          if(a != i && b != i && !Emptied(i,delta) && i!=k) /*my modified version*/
          /*if(a != i && b != i && !Emptied(i,delta))*/ /*original version*/
            lamda=lamda + (Variance(b,i,delta) - Variance(a,i,delta));
	}
      lamda=0.5 + lamda/(2*(r-2)*vab);             
    }                                              /* Formula (9) and the  */
  if(lamda > 1.0)                                /* constraint that lamda*/
    lamda = 1.0;                             /* belongs to [0,1]     */
  if(lamda < 0.0)
    lamda=0.0;
  return(lamda);
}

/******************************************************************/
/*Get_r gets the number of taxa that have not yet been aggregated in a
given distance matrix
INPUTS
float **delta : the distance matrix for a given conditioning genome
int n : total number of original taxa (including conditioning genome
int k : the conditioning genome
OUTPUTS
int r: the number of taxa (excluding the conditioning genome) in the matrix*/
/*****************************************************************/
int Get_r(float ** delta, int n, int k) /*new function for BIONJ_cond*/
{
int i,r;
r=0;
for(i=1;i<=n;i++)
	{
 	if(i!=k)
		r+=(!Emptied(i,delta));
	}
return(r);
}

/*******************************************************/
/*Get_newa finds where a taxon has gone.  For example, if we picked a and b to aggregate, but a has been emptied in some matrices at
a previous step, we need to find the index where a has been aggregated to
INPUTS
int a : index of taxon to aggregate
AGGPT *apptrs : array of pointers
OUTPUTS
int newa : where taxon a has been aggregated to*/
/*******************************************************/
int Get_newa(int a, AGGPT *aggptrs)/*new function for BIONJ_cond*/
{
int newa,ended;
AGGPT *which;

which=aggptrs[a].newnode;
ended=0;
while(!(ended))
{
	if (which->newnode!=NULL)
		which=which->newnode;
	else
		ended=1;
}
newa=which->a;
return(newa);
}

/***********************************************************************************/
/*Set_newa: decide whether one of the pair of taxa we chose to aggregate has already been aggregated in this distance matrix.
If it has, find where that taxon went and get the index for it
INPUTS
int *newa: array of taxon to aggregate in each conditioning genome
int a: the taxon we want to aggregate
AGGPT *apptrs: list of places where taxa have gone
float ***delta: set of all distance matrices
int n: number of taxa
OUTPUTS
newa is set to the value we need for each conditioning genome*/
/**********************************************************************************/	  
 void Set_newa(int *newa, int a, AGGPT *aggptrs, float ***delta, int n) /*new function for BIONJ_cond*/
 {
 int k;
 
 for (k=0;k<=n;k++)
	 {
	  newa[k]=a;
	  if (Emptied(newa[k],delta[k]))
	  {
		newa[k]=Get_newa(a, aggptrs);
	  }
	}
}




/************************************************
display_tree
list the subtrees connected to each node in the tree pointers
INPUTS
int n: number of taxa
int r: iteration of aggregation step
POINTERS *trees: tree structure
OUTPUTS
none
************************************************/

void display_tree(int n, int r, POINTERS *trees)
{
int i;
printf("r=%d\n",r);
for(i=1;i<=n;i++)
	{
	printf("i=%d\n",i);
	display_subtree(i, trees);
	}
 printf("\n");
 printf("\n");
}

/******************************************
display_subtree
displays the taxa in the subtree for each taxon
INPUTS
int i: taxon number
POINTERS *trees: tree structure
OUTPUTS
none
******************************************/
void display_subtree(int i, POINTERS *trees)
{
  WORD *parcour;
  parcour=trees[i].head;
  while(parcour != NULL)
    {
      printf("%s",parcour->name);
      parcour=parcour->suiv;
    }
  printf("\n");
}

void print_distcheck(int n, float ***delta)
/*print the set of distance matrices to check we got them right*/
{
	int i,j,k;
	for (k=1;k<=n;k++)
		{
		for (i=1;i<=n;i++)
		{
			for (j=0;j<=n;j++)
			{
				printf("%3.3f ",delta[k][i][j]);
			}
			printf("\n");
		}
		printf("\n");
	}
}

void print_abchoices(int *newa, int*newb, int n)
/*print the set of choices of taxa to aggregate for each distance matrix*/
{
int k;
printf("newa: ");
for(k=0;k<=n;k++)
	{
	printf("%d ",newa[k]);
	}
printf("\n");

printf("newb: ");
for(k=0;k<=n;k++)
	{
	printf("%d ",newb[k]);
	}
printf("\n");


}

/**************************************/	

/**************************************/	
/*get_var_weight
new function for modified bionj
weight for a vote from a given distance matrix
inputs:
float **delta: the distance matrix
int n: number of taxa
int k: which conditioning genome this is
int *cgsizes: size of each conditioning genome*/
float get_var_weight(float **delta, int n, int k, int *cgsizes)
{
	float v,w;
	int i,j,rk;

	w=cgsizes[k]; /*number of genes in this conditioning genome*/
	v=0.0;
	for(i=1;i<=n;i++){
		for(j=i+1;j<=n;j++){
			if(!Emptied(i,delta) && !Emptied(j,delta) && (i!=k) && (j!=k)){
				v+=Variance(i,j,delta);/*variance of d_{ij}*/
			}
		}
	}
	/*add covariance of d_{ij} and d_{ih} under tree-like variance assumption*/
	rk=Get_r(delta,n,k); /*number of nodes left in this matrix*/
	v=v*(rk-1);/*this accounts for covariances under the tree-like variance assumption*/
	/*if variances not tree-like, would need to have v+=cov(d{ij},d_{ih}) inside the loop below*/
/*	for(i=1;i<=n;i++){
		for(j=1;j<=n;j++){
			if(!Emptied(i,delta) && !Emptied(j,delta) && (i!=k) && (j!=k) &&(i!=j)){
				for(h=1;h<=n;h++){
					if(!Emptied(h,delta) && (h!=k) && (h!=i) && (h!=j)) {
						v+=0.5*(Variance(i,j,delta)+Variance(i,h,delta)-Variance(j,h,delta));
					}
				}
			}
		}
	}
*/	
	w= w/v;
	return w;
}
/**************************************/	

/**************************************/	
/*get_cgsizes
read the conditioning genome sizes from a file
new function for modified BIONJ
inputs:
int *cgsizes: number of genes in each conditioning genome
int n: number of taxa
FILE *slfile: file to read conditioning genome sizes from*/
void get_cgsizes(int *cgsizes, int n, FILE *slfile)
{
	int i,temp;
	for(i=1;i<=n;i++) {
		fscanf(slfile,"%d",&temp);
		cgsizes[i]=temp;
	}
}
/**************************************/	
